import multiprocessing
from functools import partial
import os
import time

def calculate_square_with_prefix(number, prefix):
    """
    计算一个数字的平方，并添加一个前缀。
    模拟一个耗时操作。
    """
    pid = os.getpid() # 获取当前进程ID
    print(f"进程ID: {pid} 正在处理 {prefix} - {number}...")
    time.sleep(0.1) # 模拟耗时操作
    result = number * number
    print(f"进程ID: {pid} 完成了 {prefix} - {number}，结果是 {result}")
    return result

if __name__ == "__main__":
    # 任务数据列表
    numbers_to_process = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]

    # 我们想要固定的参数值
    my_prefix = "计算任务"

    # 使用 functools.partial 来“冻结”函数中的 prefix 参数。
    # 这创建了一个新的函数，它只需要一个参数 (number)。
    # 这样就可以和 pool.map() 配合使用。
    # 新函数：partial_calculate_func(number)
    partial_calculate_func = partial(calculate_square_with_prefix, prefix=my_prefix)

    # 确定要使用的进程数，这里设置为4个
    process_num = 4
    print(f"主进程ID: {os.getpid()}")
    print(f"开始使用 {process_num} 个进程处理 {len(numbers_to_process)} 个任务...")

    # 使用 with 语句来创建进程池，确保它在使用后被正确关闭
    with multiprocessing.Pool(processes=process_num) as pool:
        # pool.map() 会将 numbers_to_process 列表中的每个元素，
        # 传递给 partial_calculate_func 函数，并并行执行。
        # 结果会按照输入列表的顺序返回。
        results = pool.map(partial_calculate_func, numbers_to_process)

    print("\n所有任务已完成！")
    print("最终结果列表:", results)
